package no.rwr.image;

import java.awt.image.*;
import no.rwr.data.Buffers;

public class ColorConvertOp implements RasterOp {

    protected ColorModel srcCM;
    protected ColorModel destCM;

    public ColorConvertOp(ColorModel srcCM, ColorModel destCM) {
	this.srcCM  =  srcCM;
	this.destCM = destCM;
    }
    

    public WritableRaster filter(Raster src, WritableRaster dest) {
	if (dest == null) dest = createCompatibleDestRaster(src);

	int x1 = src.getMinX();
	int y1 = src.getMinY();
	int w = src.getWidth();
	int h = src.getHeight();
	int x2 = x1+w;
	int y2 = y1+h;

	System.out.println("performing color convertion");  // DEBUG
	Object arrayIn  = null;
	Object arrayOut = null;
	for (int y=y1; y<y2; y++) {
	    System.out.println("y="+y);                     // DEBUG
	    for (int x=x1; x<x2; x++) {
		int rgb;
		try {
		    arrayIn = src.getDataElements(x, y, arrayIn);
		} catch (ArrayIndexOutOfBoundsException aioobe) {
		    String msg = "During reading of ("+x+","+y+"):"+aioobe;
		    throw new ArrayIndexOutOfBoundsException(msg);
		}
		try {
		    rgb = srcCM.getRGB(arrayIn);
		} catch (ArrayIndexOutOfBoundsException aioobe) {
		    String msg = "During --->RGB ("+x+","+y+"):"+aioobe;
		    throw new ArrayIndexOutOfBoundsException(msg);
		}
		
		try {
		    arrayOut = destCM.getDataElements(rgb, arrayOut);
		} catch (ArrayIndexOutOfBoundsException aioobe) {
		    String msg = "During RGB---> ("+x+","+y+"):"+aioobe;
		    throw new ArrayIndexOutOfBoundsException(msg);
		}
		try {
		    dest.setDataElements(x, y, arrayOut);
		} catch (ArrayIndexOutOfBoundsException aioobe) {
		    String msg = "During writing of ("+x+","+y+"):"+aioobe;
		    throw new ArrayIndexOutOfBoundsException(msg);
		}
	    }
	}

	return dest;	
    }

    public WritableRaster createCompatibleDestRaster(Raster src) {

	/* FIXME: Maybe we should explicitly create a raster with a
	   tightly pixel packed sample model, rather than assuming
	   that the createCompatibleWritableRaster() method in Raster
	   will create one. */

	return src.createCompatibleWritableRaster();
    }
}
