package no.rwr.image;
import java.awt.image.*;
import no.rwr.util.*;

public abstract class RenderCSMToSPPSM implements RasterOp {

    static final int SOURCE_BIT_WIDTH = 8;
    static final int LUT_SIZE = 1 << SOURCE_BIT_WIDTH;
    static final int LUT_MASK = LUT_SIZE - 1;
    
    static final int NUM_BANDS = 3;

    public WritableRaster filter(Raster src, WritableRaster dest) {
    	if (dest == null) dest = createCompatibleDestRaster(src);

	DataBuffer srcDB  =  src.getDataBuffer();
	DataBuffer destDB = dest.getDataBuffer();

	ComponentSampleModel  srcSM =
	    (ComponentSampleModel) src.getSampleModel();

	SinglePixelPackedSampleModel destSM =
	    (SinglePixelPackedSampleModel) dest.getSampleModel();
	
	// Calculate offset to data in the underlying arrays

	int  srcScanlineStride = srcSM.getScanlineStride();
	int destScanlineStride = destSM.getScanlineStride();

	int  srcX =  src.getMinX() -  src.getSampleModelTranslateX();
	int  srcY =  src.getMinY() -  src.getSampleModelTranslateY();
	int destX = dest.getMinX() - dest.getSampleModelTranslateX();
	int destY = dest.getMinY() - dest.getSampleModelTranslateY();

	// FIXME: assume always three
	int numBands = srcSM.getNumBands();

	/* We can't use getOffset(x, y) from the sample model since we
	   don't want the band offset added in. */
	
	int srcOffset = 
	    numBands*srcX + srcScanlineStride*srcY +    // from sample model
	    srcDB.getOffset();                          // from data buffer

	int destOffset = 
	    srcX + srcScanlineStride*srcY +             // from sample model
	    srcDB.getOffset();                          // from data buffer


	// Determine how much, and how many segments to extract.

	int rowSize = src.getWidth();
	int h = src.getHeight();

	if (((rowSize*numBands) == srcScanlineStride) &&
	    (rowSize == destScanlineStride)) {
	    // collapse scan line blits to one large blit.
	    //System.out.println("scan lines sweeps collapsed"); // DEBUG
	    
	    rowSize *= h;
	    h = 1;
	}

	render(srcDB, destDB, srcOffset, destOffset,
	       h, rowSize, srcScanlineStride, destScanlineStride);

	return dest;
    }
    
    protected abstract void render(DataBuffer srcDB, DataBuffer destDB,
				   int srcOffset, int destOffset,
				   int sweeps, int sweepSize,
				   int srcSweepStride, int destSweepStride);
    /*
	byte[]   srcArray = ((DataBufferByte) srcDB).getData();
	short[] destArray = ((DataBufferUShort) destDB).getData();
    */

    public WritableRaster createCompatibleDestRaster(Raster src) {

	/* FIXME: Maybe we should explicitly create a raster with a
	   tightly pixel packed sample model, rather than assuming
	   that the createCompatibleWritableRaster() method in Raster
	   will create one. */

	return src.createCompatibleWritableRaster();
    }
}
