
#include <X11/Xlib.h>
#include <X11/Xutil.h> // for XDestroyImage(...)

#include <cni.h>
#include <gnu/gcj/RawData.h>

#include <java/lang/OutOfMemoryError.h>

#include "no/rwr/x/Display.h"
#include "no/rwr/x/Screen.h"
#include "no/rwr/x/Visual.h"
#include "no/rwr/x/XImage.h"

void no::rwr::x::XImage::init(Visual* visual, jint depth,
			      jint format, jint xoffset,
			      jint width, jint height,
			      jint bitmapPad, jint bytesPerLine) {
    ::Display* dpy = (::Display*) visual->display->display;
    ::Visual* vis = (::Visual*) visual->getVisualStructure();

     char* data = 0; // no preallocated data
    
     ::XImage* ximage = XCreateImage(dpy, vis, depth, format, xoffset,
				     data,
				     width, height, 
				     bitmapPad,
				     bytesPerLine
				     );
     if (ximage == 0) {
	 jstring errorMessage = JvNewStringLatin1("XCreateImage failed");
	 JvThrow(new java::lang::OutOfMemoryError(errorMessage));
     }

     // Can this be checked at compile time?
     jshort endianTestShort[] = { 1 };
     jbyte* endianTestByte = reinterpret_cast<jbyte*>(endianTestShort);

     jint byteOrder;
     if (endianTestByte[0] == 1) {
	 // little endian machine
	 byteOrder = LEAST_SIGNIFICANT_B_FIRST_ORDER;
     } else {
	 // big endian machine
	 byteOrder = MOST_SIGNIFICANT_B_FIRST_ORDER;
     }

     if (byteOrder != ximage->byte_order) {
	 ximage->byte_order = byteOrder;
	 XInitImage(ximage);
     }

     structure = reinterpret_cast<gnu::gcj::RawData*>(ximage);
    // Notice that no image data has been allocated at this point

}


void no::rwr::x::XImage::init(Visual* visual, 
			      jint width,
			      jint height) {
    int depth = visual->getDepth();
    
    int format = ZPixmap; // Chunky, not planar.
    int offset = 0;
    int bitmapPad = 32; // FIXME, don't hardcode this
    int bytesPerLine = 0; // Let the server figure it out

    init(visual, depth, format, offset, width, height, bitmapPad,
	 bytesPerLine);
}


void no::rwr::x::XImage::internalSetData(jbyteArray data, jint offset) {
    ::XImage* ximage = (::XImage*) structure;
    ximage->data = reinterpret_cast<char*>(elements(data)+offset);
}

void no::rwr::x::XImage::internalSetData(jshortArray data, jint offset) {
    ::XImage* ximage = (::XImage*) structure;
    ximage->data = reinterpret_cast<char*>(elements(data)+offset);
}

void no::rwr::x::XImage::internalSetData(jintArray data, jint offset) {
    ::XImage* ximage = (::XImage*) structure;
    ximage->data = reinterpret_cast<char*>(elements(data)+offset);
}


void no::rwr::x::XImage::finalize() {
    ::XImage* ximage = (::XImage*) structure;
    if (ownsData) {
	delete ximage->data;
    }
    ximage->data = 0; // Never allow XLib to free the data allocation.
    dataRef = 0;
    XDestroyImage(ximage);
}

jint no::rwr::x::XImage::getWidth() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->width;
}

jint no::rwr::x::XImage::getHeight() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->height;
}

jint no::rwr::x::XImage::getDepth() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->depth;
}

jint no::rwr::x::XImage::getFormat() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->format;
}

jint no::rwr::x::XImage::getXOffset() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->xoffset;
}

jint no::rwr::x::XImage::getImageByteOrder() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->byte_order;
}
jint no::rwr::x::XImage::getBitmapBitOrder() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->bitmap_bit_order;
}
jint no::rwr::x::XImage::getBitmapUnit() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->bitmap_unit;
}
jint no::rwr::x::XImage::getBitmapPad() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->bitmap_pad;
}



jint no::rwr::x::XImage::getBytesPerLine() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->bytes_per_line;
}
jint no::rwr::x::XImage::getBitsPerPixel() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->bits_per_pixel;
}


// True/Direct Color specific:

jint no::rwr::x::XImage::getRedMask() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->red_mask;
}
jint no::rwr::x::XImage::getGreenMask() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->green_mask;
}
jint no::rwr::x::XImage::getBlueMask() {
    ::XImage* ximage = (::XImage*) structure;
    return ximage->blue_mask;
}


void no::rwr::x::XImage::setPixel(jint x, jint y, jint pixel) {
    ::XImage* ximage = (::XImage*) structure;
    XPutPixel(ximage, x, y, pixel);
}
